 /**Classe per la memorizzazione dei punti 2D
@author Agostini, Carlon, De Cassai
@version 0.02 Versione Abbozzata WIP 03GIU07
@version 0.03 Versione Abbozzata aggiunto metodo equals WIP 03GIU07
@version 0.04 Versione Abbozzata aggiunto metodo collineare sub mul WIP 04GIU07
*/

import java.lang.Math;

/**Classe per descrivere un punto nel piano*/
class Point2D
{
	private int x,y;
	
	
	/**Costruttore
	   @param int Il valore di X
	   @param int Il valore di Y*/
	Point2D (int x, int y)
	{	this.x = x;
		this.y = y;
	}//Point2D


	/**Costruttore di default*/
	Point2D ()
	{	x=y=0;
	}//Point2D


	/**Ritorna il valore di X
	   @return Il valore di X*/	
	public int getX ()
	{	return x;	}//getX


	/**Ritorna il valore di Y
	   @return Il valore di Y*/
	public int getY ()
	{	return y;	}//getY


	/**Imposta il nuovo valore di X.
	   @param int Il nuovo valore che si vuole dare a X
	   @return Il nuovo valore di X*/
	public int setX (int x)
	{	this.x = x;
		return getX();
	}//setX


	/**Imposta il nuovo valore di Y.
	   @param int Il nuovo valore che si vuole dare a Y
	   @return Il nuovo valore di Y*/
	public int setY (int y)
	{	this.y = y;
		return getY();
	}//setY


	/**Clona l'oggetto creato
	   @return Il clone dell'oggetto*/
	public Point2D clone ()
	{	return new Point2D(x,y);
	}//clone


	/**Ritorna la stringa di descrizione dell'oggetto.
	   @return La stringa di descrizione.*/
	public String toString ()
	{	return "[" + x + "," + y + "]";
	}//toString


	/**Calcola la distanza (in pixel) tra due punti.
	   Corrisponde al massimo tra il valore assoluto della differenza delle coordinate X e quelle Y.
	   @return La distanza tra i punti*/
	protected static int distance (Point2D a, Point2D b)
	{	return Math.max(Math.abs(a.getX()-b.getX()),Math.abs(Math.abs(a.getY()-b.getY())));
	}//distance


	/**Calcola la distanza EUCLIDEA (in pixel) tra due punti.
	   @return La distanza tra i punti*/
	public static int EuclideanDistance (Point2D a, Point2D b)
	{	return (int)Math.sqrt( (a.getX()-b.getX())  * (a.getX()-b.getX())+(a.getY()-b.getY())*(a.getY()-b.getY()));
	}//distance
	

	/**Calcola il baricentro di un array di punti Point2D.
	   @param Point2D[] I punti sul quale si calcoler il baricentro.
	   @return Il baricentro dei punti.*/
	public static Point2D centre (Point2D [] p)
	{	int x=0, y=0;
		//Se non ci sono punti, il baricentro  un punto fittizio.
		if (p==null || p.length==0) return new Point2D(-1,-1);
		for (int i=0; i<p.length; i++)
		{	x += p[i].getX();
			y += p[i].getY();
		}//for
		x /= p.length;
		y /= p.length;
		return new Point2D(x,y);
	}//centre


	/*Calcola se due punti son equivalenti
		@param Point2D il punto da verificare
		@return il risultato del confronto*/
	public boolean equals(Point2D p)
	{	return (x==p.getX()&&y==p.getY());
	}//equals


	/**Calcola la retta di regressione dei punti dati.
	   @param Point2D Primo punto.
	   @param Point2D Secondo punto.
	   @param Point2D Terzo punto.
	   @return int[][] Matrice 2x1. La retta  y=mx+q. Il primo elemento  m. Il secondo  q.*/
	public static double[][] regressionRect (Point2D a, Point2D b, Point2D c)
	{	if (a==null || b==null || c==null) return null;
		double [][] matrice = new double [2][2];
		double [][] tmp3 = new double [2][1];
		
		//Calcolo i valori temporanei della matrice
		double tmp1 = Math.pow(a.getX(),2) + Math.pow(b.getX(),2) + Math.pow(c.getX(),2);
		double tmp2 = a.getX() + b.getX() + c.getX();
		double delta = 3 * tmp1 - Math.pow(tmp2,2);
		//Calcolo i valori della matrice
		matrice[0][0] = 3 / delta;
		matrice[1][0] = matrice[0][1] = - tmp2 / delta;
		matrice[1][1] = tmp1 / delta;
		//Calcolo i calori della seconda matrice
		tmp3[0][0] = a.getX()*a.getY()+ b.getX()*b.getY() + c.getX()*c.getY();
		tmp3[1][0] = a.getY() + b.getY() + c.getY();
		
		//Matrice*tmp3=regressione ==> y=mx+q ==> m=regressione[0][0] q=regressione[1][0]
		return Matrix.mul(matrice,tmp3);
	}//regressionRect


	/**Dati tre punti, riconosce se collineari.
	   @param Point2D Primo punto su cui analizzare la collinearit.
	   @param Point2D Secondo punto su cui analizzare la collinearit.
	   @param Point2D Terzo punto su cui analizzare la collinearit.
	   @return Errore medio dei punti rispetto alla retta di regressione.*/
	public static double collineare (Point2D a, Point2D b, Point2D c)
	{	if (a==null || b==null || c==null) return -1;
		double regressione [][] = regressionRect(a,b,c);
		
		//Calcolo dell'errore
		double errore = Math.pow (regressione[0][0] * (double)a.getX() + regressione[1][0] - (double)a.getY() , 2);
		errore += Math.pow (regressione[0][0] * (double)b.getX() + regressione[1][0] - (double)b.getY() , 2);
		errore += Math.pow (regressione[0][0] * (double)c.getX() + regressione[1][0] - (double)c.getY() , 2);
		return Math.sqrt(errore)/3;
	}//collinear


	/**Dato un array di punti, riconosce se sono collineari.
	   @param Point2D [] Punti sul quale applicare il riconoscimento di collinearit
	   @return Gli indici degli elementi dell'array che hanno portato al minimo errore.*/
	public static int[] collineare (Point2D[] p)
	{	double errore, errore_min = Double.POSITIVE_INFINITY ;
		int ris[] = new int [3];
		//Calcolo tutte le combinazioni dell'array p di classe 3: ho bisogno di conoscere la collinearit di tre punti.
		for (int i=0; i<p.length-2; i++)
			for (int j=i+1; j<p.length-1; j++)
				for (int k=j+1; k<p.length; k++)
					if ((errore = collineare(p[i],p[j],p[k])) < errore_min)
					{	errore_min = errore;
						ris[0] = i;
						ris[1] = j;
						ris[2] = k;
					}//if
//		System.out.println (ris[0] + " " + ris[1] + " " + ris[2]);
//		System.out.println (errore_min);
		return ris;
	}//collineare


	/**Moltiplica le coordinate per uno scalare
	   @param int Lo scalare.*/
	public void mul (int scalar)
	{	setX(getX()*scalar);
		setY(getY()*scalar);
	}//mul


	/**Esegue la differenza del punto con il parametro passato.
	   @param Point2D Il punto sottratto.*/
	public void sub (Point2D s)
	{	setX(getX()-s.getX());
		setY(getY()-s.getY());
	}//sub
	
	/**Ruoto il punto rispetto l'origine di alfa
	@param int angolo di rotazione in gradi*/
	public void rotate(int alfa){
		double a=Math.PI*(alfa/180);
		
		double c=Math.cos(a); double s=Math.sin(a);
	//	System.out.print(" sin "+s+" cos "+c);
		int tx=(int)(c*x-s*y);
		y=(int)(s*x+c*y);
		x=tx;
	}
	
		public void sum (Point2D s)
	{	setX(getX()+s.getX());
		setY(getY()+s.getY());
	}//sub

}//Point2D