 /**Classe per l'analisi delle posizioni dei punti della mano nell'immagine
@author Agostini, Carlon, De Cassai
@version 0.02 Versione Abbozzata WIP 03GIU07
@version 0.03 Versione Abbozzata Stabile, aggiunto Sensor Distance e Rotation WIP 03GIU07

*/

import java.util.Vector;
//import java.util.Math;

public class PointsAnalyzer
{
	//Gli insiemi pi piccoli di questo valore vengono ignorati
	private static final int SOGLIA_DIM_SET = 3;


	/**Crea un array di punti che corrispondono ai punti bianchi dell'ImageStreaming.
	   @param ImageStreaming L'ImageStreaming dalla quale estrarre i punti bianchi.
	   @return L'array di Point2D con le coordinate dei punti accesi.*/
	private static Point2D[] createWhitePoint (ImageStreaming imm)
	{
		int [][] space = new int [imm.getHeight()][imm.getWidth()];
		Vector points_vec = new Vector();

		//Riempio i valori della matrice con i colori dell'ImageStreaming e anche il vettore dei punti bianchi
		for (int i=0; i<imm.getHeight(); i++)
			for (int j=0; j<imm.getWidth(); j++)
				if ((space[i][j]=imm.getRGB(j,i))==-1)
					points_vec.add(new Point2D(j,i));
		//Converto il Vettore in Array
		Object [] tmp_array = points_vec.toArray();
		Point2D [] points_array = new Point2D[tmp_array.length];
		for (int i=0; i<tmp_array.length; i++)
			points_array[i]=(Point2D)tmp_array[i];
			
		return points_array;
	}//createImmMatrix



	/**Crea gli insiemi di punti contigui nell'ImageStreaming.
	   @param String Il percorso dell'ImageStreaming da aprire.
	   @return L'array di punti Point2D dei centri degli insiemi.*/
	public static Point2D[] createSets (ImageStreaming imm)
	{	Point2D whitePoints[] = createWhitePoint(imm);  //Creo i punti bianche dall'ImageStreaming
		int length = whitePoints.length; //Numero di punti non ancora assegnati
		SetPoint2D set; //Insieme di punti
		Vector centres = new Vector (); //Creo l'array dei centri delle luci
		//Creo gli insiemi e i centri degli insiemi
		for (int i=0; length>0; i++)
		{	set = new SetPoint2D();
			length = set.buildSet(whitePoints, length);
			//Considero solo gli insiemi con un numero di elementi superiori alla soglia
			if (set.size()>=SOGLIA_DIM_SET) centres.add (set.getCentre());
//			System.out.println (set);
		}//for
		
		return vectorToPoint2D(centres);
	}//createSets



	/**Converte un Vector di Point2D in un array di Point2D.
	   @param Vector Il vettore di Point2D da convertire.
	   @return L'array di Point2D*/
	private static Point2D[] vectorToPoint2D (Vector v)
	{	Point2D[] p = new Point2D [v.size()];
		int size = v.size();
		for (int i=0; i<size; i++)
		{	p[i] = (Point2D) v.firstElement(); //Prendo il primo elemento e lo metto nell'array
			v.remove(0); //Rimuovo il primo elemento del Vector
		}//while
		return p;
	}//vector2Point2D



	/**Converte un Vector di SetPoint2D in un array di SetPoint2D.
	   @param Vector Il vettore di SetPoint2D da convertire.
	   @return L'array di SetPoint2D*/
	private static SetPoint2D[] vectorToSetPoint2D (Vector v)
	{	SetPoint2D[] s = new SetPoint2D [v.size()];
		int size = v.size();
		for (int i=0; i<size; i++)
		{	s[i] = (SetPoint2D) v.firstElement(); //Prendo il primo elemento e lo metto nell'array
			v.remove(0); //Rimuovo il primo elemento del Vector
		}//while
		return s;
	}//vector2Point2D
	
	/*Rende una stima della distanza dal sensore.
		@param Point2D il primo punto
		@param point2D il secondo punto
		@return la distanza stimata, se i punti coincidono rendo 0*/
	public static int sensorDistance(Point2D a,Point2D b){
		if (!a.equals(b))
		return (10000/Point2D.EuclideanDistance(a,b));
		return 0;
	}
	
	/*Calcolo il coefficiente angolare della retta passante per i punti ab, espressa come [0..360]. non gestisco il caso di punti coincidenti
		@param Point2D il primo punto
		@param point2D il secondo punto
		@return la rotazione in angoli, senza decimali*/
	public static int rotation(Point2D a,Point2D b){
		if (b.getX()==a.getX()){
			if (b.getY()>a.getY())
				return 90;
			return 270;
		}
		if (b.getY()==a.getY()){
			if (b.getX()>a.getX())
				return 0;
			return 180;
		}
		double d=Point2D.EuclideanDistance(a,b);
		double s=b.getY()-a.getY();	s/=d;//queste righe intermedie servon per evitare l'arrotondamento a 0
		//IL SENO HA UN MENO A CAUSA DELLE COORDINATE DELLE IMMAGINI INVERTITE IN JAVA-.-'
		s=-(180*Math.asin(s))/Math.PI;
	//	System.out.print(a+" "+b);
		double c=b.getX()-a.getX(); c/=d;
		c=(180*Math.acos(c))/Math.PI;
	//	System.out.println("  Sn: "+s+" Cs: "+c);
		
		
		if (c<90) return (int)(s+360)%360;
		return (int)((180-s)+360)%360;
	}
	

/**Calcolo la distanza di un punto da una data retta
	@param Point2D il punto
	@param double il coefficiente angolare della retta
	@param double l'offset della retta*/
	public static double pointRectDistance(Point2D p, double m, double q){
	
		return (double)Math.abs((p.getY()-m*p.getX()-q)/(Math.sqrt(1+m*m)));
	
	}
	
/**Rendo i parametri della retta passante per i due punti dati a parametro
	@param Point2D il primo punto
	@param Point2D il secondo punto
	@return double[] primo elemento coefficente angolare, secondo offset*/
	public static double[] twoPointRect(Point2D a, Point2D b){
		double[] ret=new double[2];
		ret[0]=(b.getY()-a.getY())/(b.getX()-a.getX());
		ret[1]=-ret[0]*a.getX()+a.getY();
		return ret;
	}

}//PointsAnalyzer 
